@@ -297,7 +297,18 @@ Rcpp::List hmc_singolo(const arma::vec& theta0,
       alpha_tmp = std::exp(lsw2 - lsw);
       
       //update the proposed values
-      for(unsigned int i = 0; i < K; i++){
+      
+      //biased for the first
+      if(arma::randu() < alpha_tmp){
+        theta.subvec(0,d-1) = theta_p;
+      }
+      
+      //update the log sum of the multinomial weights
+      lsw = arma::log_add_exp(lsw,lsw2);
+      alpha_tmp = std::exp(lsw2 - lsw);
+      
+      //uniform for the others
+      for(unsigned int i = 1; i < K; i++){
         if(arma::randu() < alpha_tmp){
           theta.subvec(i*d,(i+1)*d-1) = theta_p;
         }
@@ -311,7 +322,18 @@ Rcpp::List hmc_singolo(const arma::vec& theta0,
       alpha_tmp = std::exp(lsw2 - lsw);
       
       //update the proposed values
-      for(unsigned int i = 0; i < K; i++){
+      
+      //biased for the first
+      if(arma::randu() < alpha_tmp){
+        theta.subvec(0,d-1) = theta_m;
+      }
+      
+      //update the log sum of the multinomial weights
+      lsw = arma::log_add_exp(lsw,lsw2);
+      alpha_tmp = std::exp(lsw2 - lsw);
+      
+      //uniform for the others
+      for(unsigned int i = 1; i < K; i++){
         if(arma::randu() < alpha_tmp){
           theta.subvec(i*d,(i+1)*d-1) = theta_m;
         }
@@ -319,9 +341,6 @@ Rcpp::List hmc_singolo(const arma::vec& theta0,
       
     }
     
-    //update the log sum of the multinomial weights
-    lsw = arma::log_add_exp(lsw,lsw2);
-    
     //increase trajectory length
     step_length++;
   }
@@ -391,7 +410,18 @@ Rcpp::List hmc_singolo(const arma::vec& theta0,
       alpha_tmp = std::exp(lsw2 - lsw);
       
       //update the proposed values
-      for(unsigned int i = 0; i < K; i++){
+      
+      //biased for the first
+      if(arma::randu() < alpha_tmp){
+        theta.subvec(0,d-1) = theta_p;
+      }
+      
+      //update the log sum of the multinomial weights
+      lsw = arma::log_add_exp(lsw,lsw2);
+      alpha_tmp = std::exp(lsw2 - lsw);
+      
+      //uniform for the others
+      for(unsigned int i = 1; i < K; i++){
         if(arma::randu() < alpha_tmp){
           theta.subvec(i*d,(i+1)*d-1) = theta_p;
         }
@@ -405,7 +435,18 @@ Rcpp::List hmc_singolo(const arma::vec& theta0,
       alpha_tmp = std::exp(lsw2 - lsw);
       
       //update the proposed values
-      for(unsigned int i = 0; i < K; i++){
+      
+      //biased for the first
+      if(arma::randu() < alpha_tmp){
+        theta.subvec(0,d-1) = theta_m;
+      }
+      
+      //update the log sum of the multinomial weights
+      lsw = arma::log_add_exp(lsw,lsw2);
+      alpha_tmp = std::exp(lsw2 - lsw);
+      
+      //uniform for the others
+      for(unsigned int i = 1; i < K; i++){
         if(arma::randu() < alpha_tmp){
           theta.subvec(i*d,(i+1)*d-1) = theta_m;
         }
@@ -413,9 +454,6 @@ Rcpp::List hmc_singolo(const arma::vec& theta0,
       
     }
     
-    //update the log sum of the multinomial weights
-    lsw = arma::log_add_exp(lsw,lsw2);
-    
     //increase trajectory length
     step_length++;
   }
@@ -485,7 +523,18 @@ Rcpp::List hmc_singolo(const arma::vec& theta0,
       alpha_tmp = std::exp(lsw2 - lsw);
       
       //update the proposed values
-      for(unsigned int i = 0; i < K; i++){
+      
+      //biased for the first
+      if(arma::randu() < alpha_tmp){
+        theta.subvec(0,d-1) = theta_p;
+      }
+      
+      //update the log sum of the multinomial weights
+      lsw = arma::log_add_exp(lsw,lsw2);
+      alpha_tmp = std::exp(lsw2 - lsw);
+      
+      //uniform for the others
+      for(unsigned int i = 1; i < K; i++){
         if(arma::randu() < alpha_tmp){
           theta.subvec(i*d,(i+1)*d-1) = theta_p;
         }
@@ -499,17 +548,25 @@ Rcpp::List hmc_singolo(const arma::vec& theta0,
       alpha_tmp = std::exp(lsw2 - lsw);
       
       //update the proposed values
-      for(unsigned int i = 0; i < K; i++){
+      
+      //biased for the first
+      if(arma::randu() < alpha_tmp){
+        theta.subvec(0,d-1) = theta_m;
+      }
+      
+      //update the log sum of the multinomial weights
+      lsw = arma::log_add_exp(lsw,lsw2);
+      alpha_tmp = std::exp(lsw2 - lsw);
+      
+      //uniform for the others
+      for(unsigned int i = 1; i < K; i++){
         if(arma::randu() < alpha_tmp){
           theta.subvec(i*d,(i+1)*d-1) = theta_m;
         }
       }
       
     }
-    
-    //update the log sum of the multinomial weights
-    lsw = arma::log_add_exp(lsw,lsw2);
-    
+
     //increase trajectory length
     step_length++;
   }
@@ -789,7 +846,18 @@ Rcpp::List hmc_singolo(const arma::vec& theta0,
       alpha_tmp = std::exp(lsw2 - lsw);
       
       //update the proposed values
-      for(unsigned int i = 0; i < K; i++){
+      
+      //biased for the first
+      if(arma::randu() < alpha_tmp){
+        theta.subvec(0,d-1) = theta_p;
+      }
+      
+      //update the log sum of the multinomial weights
+      lsw = arma::log_add_exp(lsw,lsw2);
+      alpha_tmp = std::exp(lsw2 - lsw);
+      
+      //uniform for the others
+      for(unsigned int i = 1; i < K; i++){
         if(arma::randu() < alpha_tmp){
           theta.subvec(i*d,(i+1)*d-1) = theta_p;
         }
@@ -803,7 +871,18 @@ Rcpp::List hmc_singolo(const arma::vec& theta0,
       alpha_tmp = std::exp(lsw2 - lsw);
       
       //update the proposed values
-      for(unsigned int i = 0; i < K; i++){
+      
+      //biased for the first
+      if(arma::randu() < alpha_tmp){
+        theta.subvec(0,d-1) = theta_m;
+      }
+      
+      //update the log sum of the multinomial weights
+      lsw = arma::log_add_exp(lsw,lsw2);
+      alpha_tmp = std::exp(lsw2 - lsw);
+      
+      //uniform for the others
+      for(unsigned int i = 1; i < K; i++){
         if(arma::randu() < alpha_tmp){
           theta.subvec(i*d,(i+1)*d-1) = theta_m;
         }
@@ -811,9 +890,6 @@ Rcpp::List hmc_singolo(const arma::vec& theta0,
       
     }
     
-    //update the log sum of the multinomial weights
-    lsw = arma::log_add_exp(lsw,lsw2);
-    
     //increase trajectory length
     step_length++;
   }
@@ -876,7 +952,18 @@ Rcpp::List hmc_singolo(const arma::vec& theta0,
       alpha_tmp = std::exp(lsw2 - lsw);
       
       //update the proposed values
-      for(unsigned int i = 0; i < K; i++){
+      
+      //biased for the first
+      if(arma::randu() < alpha_tmp){
+        theta.subvec(0,d-1) = theta_p;
+      }
+      
+      //update the log sum of the multinomial weights
+      lsw = arma::log_add_exp(lsw,lsw2);
+      alpha_tmp = std::exp(lsw2 - lsw);
+      
+      //uniform for the others
+      for(unsigned int i = 1; i < K; i++){
         if(arma::randu() < alpha_tmp){
           theta.subvec(i*d,(i+1)*d-1) = theta_p;
         }
@@ -890,7 +977,18 @@ Rcpp::List hmc_singolo(const arma::vec& theta0,
       alpha_tmp = std::exp(lsw2 - lsw);
       
       //update the proposed values
-      for(unsigned int i = 0; i < K; i++){
+      
+      //biased for the first
+      if(arma::randu() < alpha_tmp){
+        theta.subvec(0,d-1) = theta_m;
+      }
+      
+      //update the log sum of the multinomial weights
+      lsw = arma::log_add_exp(lsw,lsw2);
+      alpha_tmp = std::exp(lsw2 - lsw);
+      
+      //uniform for the others
+      for(unsigned int i = 1; i < K; i++){
         if(arma::randu() < alpha_tmp){
           theta.subvec(i*d,(i+1)*d-1) = theta_m;
         }
@@ -898,9 +996,6 @@ Rcpp::List hmc_singolo(const arma::vec& theta0,
       
     }
     
-    //update the log sum of the multinomial weights
-    lsw = arma::log_add_exp(lsw,lsw2);
-    
     //increase trajectory length
     step_length++;
   }
@@ -963,7 +1058,18 @@ Rcpp::List hmc_singolo(const arma::vec& theta0,
       alpha_tmp = std::exp(lsw2 - lsw);
       
       //update the proposed values
-      for(unsigned int i = 0; i < K; i++){
+      
+      //biased for the first
+      if(arma::randu() < alpha_tmp){
+        theta.subvec(0,d-1) = theta_p;
+      }
+      
+      //update the log sum of the multinomial weights
+      lsw = arma::log_add_exp(lsw,lsw2);
+      alpha_tmp = std::exp(lsw2 - lsw);
+      
+      //uniform for the others
+      for(unsigned int i = 1; i < K; i++){
         if(arma::randu() < alpha_tmp){
           theta.subvec(i*d,(i+1)*d-1) = theta_p;
         }
@@ -977,17 +1083,25 @@ Rcpp::List hmc_singolo(const arma::vec& theta0,
       alpha_tmp = std::exp(lsw2 - lsw);
       
       //update the proposed values
-      for(unsigned int i = 0; i < K; i++){
+      
+      //biased for the first
+      if(arma::randu() < alpha_tmp){
+        theta.subvec(0,d-1) = theta_m;
+      }
+      
+      //update the log sum of the multinomial weights
+      lsw = arma::log_add_exp(lsw,lsw2);
+      alpha_tmp = std::exp(lsw2 - lsw);
+      
+      //uniform for the others
+      for(unsigned int i = 1; i < K; i++){
         if(arma::randu() < alpha_tmp){
           theta.subvec(i*d,(i+1)*d-1) = theta_m;
         }
       }
       
     }
-    
-    //update the log sum of the multinomial weights
-    lsw = arma::log_add_exp(lsw,lsw2);
-    
+
     //increase trajectory length
     step_length++;
   }
@@ -1026,10 +1140,11 @@ Rcpp::List hmc_singolo(const arma::vec& theta0,
   arma::vec theta = theta0;
   
   //calculate the value of the current potential energy
-  double U = Rcpp::as<double>(nlp(theta,args,true));
+  double U_m = Rcpp::as<double>(nlp(theta,args,true));
+  double U_p = U_m;
   
   //calculate the current value of the Hamiltonian (energy level to be explored)
-  double H0 = U + arma::sum(arma::abs(m_m));
+  double H0 = U_m + arma::sum(arma::abs(m_m));
   
   //length of the trajectory 
   unsigned int step_length = 0;
@@ -1043,19 +1158,21 @@ Rcpp::List hmc_singolo(const arma::vec& theta0,
     //should the doubling be done on the right or left?
     if(arma::randu() > 0.5){
       //continue the trajectory forward
-      leapfrog(theta_p,m_p,alpha,eps,nlp,args,U,d,idx_disc);
+      //leapfrog(theta_p,m_p,alpha,eps,nlp,args,U,d,idx_disc);
+      leapfrog(theta_p,m_p,alpha,eps,nlp,args,U_p,d,idx_disc);
       
       //biased progressive sampling
-      if(arma::randu() < 1/(1+step_length)){
-        theta = theta_p;
+      if(arma::randu() < 1.0/(1+step_length)){
+          theta = theta_p;
       }
       
     }else{
       //continue the trajectory backward
-      leapfrog(theta_m,m_m,alpha,-eps,nlp,args,U,d,idx_disc);
+      //leapfrog(theta_m,m_m,alpha,-eps,nlp,args,U,d,idx_disc);
+      leapfrog(theta_m,m_m,alpha,-eps,nlp,args,U_m,d,idx_disc);
       
       //biased progressive sampling
-      if(arma::randu() < 1/(1+step_length)){
+      if(arma::randu() < 1.0/(1+step_length)){
         theta = theta_m;
       }
     }
@@ -1093,10 +1210,11 @@ Rcpp::List hmc_singolo(const arma::vec& theta0,
   arma::vec theta = theta0;
   
   //calculate the value of the current potential energy
-  double U = Rcpp::as<double>(nlp(theta,args,true));
+  double U_m = Rcpp::as<double>(nlp(theta,args,true));
+  double U_p = U_m;
   
   //calculate the current value of the Hamiltonian (energy level to be explored)
-  double H0 = U + arma::dot(arma::abs(m_m),M_inv);
+  double H0 = U_m + arma::sum(arma::abs(m_m));
   
   //length of the trajectory 
   unsigned int step_length = 0;
@@ -1110,19 +1228,19 @@ Rcpp::List hmc_singolo(const arma::vec& theta0,
     //should the doubling be done on the right or left?
     if(arma::randu() > 0.5){
       //continue the trajectory forward
-      leapfrog(theta_p,m_p,alpha,eps,nlp,args,U,d,idx_disc,M_inv);
+      leapfrog(theta_p,m_p,alpha,eps,nlp,args,U_p,d,idx_disc,M_inv);
       
       //biased progressive sampling
-      if(arma::randu() < 1/(1+step_length)){
+      if(arma::randu() < 1.0/(1+step_length)){
         theta = theta_p;
       }
       
     }else{
       //continue the trajectory backward
-      leapfrog(theta_m,m_m,alpha,-eps,nlp,args,U,d,idx_disc,M_inv);
+      leapfrog(theta_m,m_m,alpha,-eps,nlp,args,U_m,d,idx_disc,M_inv);
       
       //biased progressive sampling
-      if(arma::randu() < 1/(1+step_length)){
+      if(arma::randu() < 1.0/(1+step_length)){
         theta = theta_m;
       }
     }
@@ -1166,10 +1284,11 @@ Rcpp::List hmc_singolo(const arma::vec& theta0,
   }
   
   //calculate the value of the current potential energy
-  double U = Rcpp::as<double>(nlp(theta0,args,true));
+  double U_m = Rcpp::as<double>(nlp(theta0,args,true));
+  double U_p = U_m;
   
   //calculate the current value of the Hamiltonian (energy level to be explored)
-  double H0 = U + arma::sum(arma::abs(m_m));
+  double H0 = U_m + arma::sum(arma::abs(m_m));
   
   //length of the trajectory 
   unsigned int step_length = 0;
@@ -1183,25 +1302,25 @@ Rcpp::List hmc_singolo(const arma::vec& theta0,
     //should the doubling be done on the right or left?
     if(arma::randu() > 0.5){
       //continue the trajectory forward
-      leapfrog(theta_p,m_p,alpha,eps,nlp,args,U,d,idx_disc);
+      leapfrog(theta_p,m_p,alpha,eps,nlp,args,U_p,d,idx_disc);
       
-      //biased progressive sampling
       //update the proposed values
+      //biased sampling for all
       for(unsigned int i = 0; i < K; i++){
-        if(arma::randu() < 1/(1+step_length)){
+        if(arma::randu() < 1.0/(1+step_length)){
           theta.subvec(i*d,(i+1)*d-1) = theta_p;
         }
       }
       
     }else{
       //continue the trajectory backward
-      leapfrog(theta_m,m_m,alpha,-eps,nlp,args,U,d,idx_disc);
+      leapfrog(theta_m,m_m,alpha,-eps,nlp,args,U_m,d,idx_disc);
       
-      //biased progressive sampling
       //update the proposed values
+      //biased sampling for all
       for(unsigned int i = 0; i < K; i++){
-        if(arma::randu() < 1/(1+step_length)){
-          theta.subvec(i*d,(i+1)*d-1) = theta_p;
+        if(arma::randu() < 1.0/(1+step_length)){
+          theta.subvec(i*d,(i+1)*d-1) = theta_m;
         }
       }
     }
@@ -1218,7 +1337,7 @@ Rcpp::List hmc_singolo(const arma::vec& theta0,
                             Rcpp::Named("E") = H0);
 }
 
-// identity matrix case
+// diagonal matrix case
 Rcpp::List hmc_singolo(const arma::vec& theta0,
                        const arma::vec& m0,
                        const Rcpp::Function& nlp,
@@ -1243,10 +1362,11 @@ Rcpp::List hmc_singolo(const arma::vec& theta0,
   }
   
   //calculate the value of the current potential energy
-  double U = Rcpp::as<double>(nlp(theta0,args,true));
+  double U_m = Rcpp::as<double>(nlp(theta0,args,true));
+  double U_p = U_m;
   
   //calculate the current value of the Hamiltonian (energy level to be explored)
-  double H0 = U + arma::dot(arma::abs(m_m),M_inv);
+  double H0 = U_m + arma::sum(arma::abs(m_m));
   
   //length of the trajectory 
   unsigned int step_length = 0;
@@ -1260,25 +1380,25 @@ Rcpp::List hmc_singolo(const arma::vec& theta0,
     //should the doubling be done on the right or left?
     if(arma::randu() > 0.5){
       //continue the trajectory forward
-      leapfrog(theta_p,m_p,alpha,eps,nlp,args,U,d,idx_disc,M_inv);
+      leapfrog(theta_p,m_p,alpha,eps,nlp,args,U_p,d,idx_disc,M_inv);
       
-      //biased progressive sampling
       //update the proposed values
+      //biased sampling for all
       for(unsigned int i = 0; i < K; i++){
-        if(arma::randu() < 1/(1+step_length)){
+        if(arma::randu() < 1.0/(1+step_length)){
           theta.subvec(i*d,(i+1)*d-1) = theta_p;
         }
       }
       
     }else{
       //continue the trajectory backward
-      leapfrog(theta_m,m_m,alpha,-eps,nlp,args,U,d,idx_disc,M_inv);
+      leapfrog(theta_m,m_m,alpha,-eps,nlp,args,U_m,d,idx_disc,M_inv);
       
-      //biased progressive sampling
       //update the proposed values
+      //biased sampling for all
       for(unsigned int i = 0; i < K; i++){
-        if(arma::randu() < 1/(1+step_length)){
-          theta.subvec(i*d,(i+1)*d-1) = theta_p;
+        if(arma::randu() < 1.0/(1+step_length)){
+          theta.subvec(i*d,(i+1)*d-1) = theta_m;
         }
       }
     }
